@@ -4,12 +4,12 @@
 /*
  * conffile.h	Defines for the conffile parsing routines.
  *
- * Version:	$Id: conffile.h,v 1.43 2007/12/27 05:41:40 aland Exp $
+ * Version:	$Id$
  *
  */
 
 #include <freeradius-devel/ident.h>
-RCSIDH(conffile_h, "$Id: conffile.h,v 1.43 2007/12/27 05:41:40 aland Exp $")
+RCSIDH(conffile_h, "$Id$")
 
 #include <stddef.h>
 #include <freeradius-devel/token.h>
@@ -43,18 +43,17 @@ typedef struct CONF_PARSER {
 /* This preprocessor trick will be useful in initializing CONF_PARSER struct */
 #define XStringify(x) #x
 #define Stringify(x) XStringify(x)
-/* And this pointer trick too */
-#ifndef offsetof
-# define offsetof(TYPE, MEMBER) ((size_t) &((TYPE *)0)->MEMBER)
-#endif
 
 void		cf_pair_free(CONF_PAIR **cp);
+int		cf_pair_replace(CONF_SECTION *cs, CONF_PAIR *cp,
+				const char *value);
 void		cf_section_free(CONF_SECTION **cp);
 int		cf_item_parse(CONF_SECTION *cs, const char *name,
 			      int type, void *data, const char *dflt);
 int		cf_section_parse(CONF_SECTION *, void *base,
 				 const CONF_PARSER *variables);
 void		cf_section_parse_free(CONF_SECTION *cs, void *base);
+const CONF_PARSER *cf_section_parse_table(CONF_SECTION *cs);
 CONF_SECTION	*cf_file_read(const char *file);
 int		cf_file_include(const char *file, CONF_SECTION *cs);
 
@@ -93,9 +92,21 @@ CONF_SECTION *cf_itemtosection(CONF_ITEM *item);
 CONF_ITEM *cf_pairtoitem(CONF_PAIR *cp);
 CONF_ITEM *cf_sectiontoitem(CONF_SECTION *cs);
 int cf_section_template(CONF_SECTION *cs, CONF_SECTION *template);
-void cf_log_err(CONF_ITEM *ci, const char *fmt, ...);
-void cf_log_info(CONF_SECTION *cs, const char *fmt, ...);
-void cf_log_module(CONF_SECTION *cs, const char *fmt, ...);
+void cf_log_err(CONF_ITEM *ci, const char *fmt, ...)
+#ifdef __GNUC__
+		__attribute__ ((format (printf, 2, 3)))
+#endif
+;
+void cf_log_info(CONF_SECTION *cs, const char *fmt, ...)
+#ifdef __GNUC__
+		__attribute__ ((format (printf, 2, 3)))
+#endif
+;
+void cf_log_module(CONF_SECTION *cs, const char *fmt, ...)
+#ifdef __GNUC__
+		__attribute__ ((format (printf, 2, 3)))
+#endif
+;
 CONF_ITEM *cf_reference_item(const CONF_SECTION *parentcs,
 			     CONF_SECTION *outercs,
 			     const char *ptr);
