@@ -4,7 +4,7 @@
  * radiusd.h	Structures, prototypes and global variables
  *		for the FreeRADIUS server.
  *
- * Version:	$Id: radiusd.h,v 1.263 2008/04/30 10:00:53 aland Exp $
+ * Version:	$Id$
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSIDH(radiusd_h, "$Id: radiusd.h,v 1.263 2008/04/30 10:00:53 aland Exp $")
+RCSIDH(radiusd_h, "$Id$")
 
 #include <freeradius-devel/libradius.h>
 #include <freeradius-devel/radpaths.h>
@@ -35,30 +35,74 @@ RCSIDH(radiusd_h, "$Id: radiusd.h,v 1.263 2008/04/30 10:00:53 aland Exp $")
 
 typedef struct auth_req REQUEST;
 
-#include <freeradius-devel/realms.h>
-
 #ifdef HAVE_PTHREAD_H
 #include	<pthread.h>
 typedef pthread_t child_pid_t;
-#define child_kill pthread_kill
-#else
-typedef pid_t child_pid_t;
-#define child_kill kill
-#endif
-
 #define NO_SUCH_CHILD_PID (child_pid_t) (0)
+#endif
 
 #ifndef NDEBUG
 #define REQUEST_MAGIC (0xdeadbeef)
 #endif
 
+/*
+ *	New defines for minimizing the size of the server, to strip
+ *	out functionality.  In order to ensure that people don't have
+ *	to re-run "configure", after "cvs update", we play some
+ *	special games with the defines.  i.e. any top-level "configure"
+ *	option should set both WITH_FOO and WITHOUT_FOO.  After a few
+ *	weeks, the WITHOUT_FOO can be deleted from the configure script.
+ */
+#ifndef WITHOUT_PROXY
+#define WITH_PROXY (1)
+#endif
+
+#ifndef WITHOUT_DETAIL
+#define WITH_DETAIL (1)
+#endif
+
+#ifndef WITHOUT_SESSION_MGMT
+#define WITH_SESSION_MGMT (1)
+#endif
+
+#ifndef WITHOUT_UNLANG
+#define WITH_UNLANG (1)
+#endif
+
+#ifndef WITHOUT_ACCOUNTING
+#define WITH_ACCOUNTING (1)
+#else
+#ifdef WITH_SESSION_MGMT
+#error WITH_SESSION_MGMT is defined, but WITH_ACCOUNTING is not.  Session management requires accounting.
+#endif
+#ifdef WITH_DETAIL
+#error WITH_DETAIL is defined, but WITH_ACCOUNTING is not.  Detail file reading requires accounting.
+#endif
+#endif
+
+#ifndef WITHOUT_DYNAMIC_CLIENTS
+#define WITH_DYNAMIC_CLIENTS (1)
+#endif
+
+#ifndef WITHOUT_STATS
+#define WITH_STATS
+#endif
+
+#ifndef WITHOUT_COMMAND_SOCKET
+#ifdef HAVE_SYS_UN_H
+#define WITH_COMMAND_SOCKET (1)
+#endif
+#endif
+
+#include <freeradius-devel/stats.h>
+#include <freeradius-devel/realms.h>
+
+
 /*
  *	See util.c
  */
 typedef struct request_data_t request_data_t;
 
-typedef struct rad_snmp_client_entry_t rad_snmp_client_entry_t;
-
 typedef struct radclient {
 	fr_ipaddr_t		ipaddr;
 	int			prefix;
@@ -72,8 +116,19 @@ typedef struct radclient {
 	char			*server;
 	int			number;	/* internal use only */
 	const CONF_SECTION	*cs;
-#ifdef WITH_SNMP
-	rad_snmp_client_entry_t *auth, *acct;
+#ifdef WITH_STATS
+	fr_stats_t		*auth;
+#ifdef WITH_ACCOUNTING
+	fr_stats_t		*acct;
+#endif
+#endif
+
+#ifdef WITH_DYNAMIC_CLIENTS
+	int			lifetime;
+	int			dynamic; /* was dynamically defined */
+	time_t			created;
+	time_t			last_new_client;
+	char			*client_server;
 #endif
 } RADCLIENT;
 
@@ -84,13 +139,25 @@ typedef struct radclient {
  */
 typedef enum RAD_LISTEN_TYPE {
 	RAD_LISTEN_NONE = 0,
+#ifdef WITH_PROXY
 	RAD_LISTEN_PROXY,
+#endif
 	RAD_LISTEN_AUTH,
+#ifdef WITH_ACCOUNTING
 	RAD_LISTEN_ACCT,
+#endif
+#ifdef WITH_DETAIL
 	RAD_LISTEN_DETAIL,
+#endif
+#ifdef WITH_VMPS
 	RAD_LISTEN_VQP,
+#endif
+#ifdef WITH_DHCP
 	RAD_LISTEN_DHCP,
-	RAD_LISTEN_SNMP,
+#endif
+#ifdef WITH_COMMAND_SOCKET
+	RAD_LISTEN_COMMAND,
+#endif
 	RAD_LISTEN_MAX
 } RAD_LISTEN_TYPE;
 
@@ -99,6 +166,7 @@ typedef enum RAD_LISTEN_TYPE {
  *	For listening on multiple IP's and ports.
  */
 typedef struct rad_listen_t rad_listen_t;
+typedef		void (*radlog_func_t)(int, int, REQUEST *, const char *, ...);
 
 #define REQUEST_DATA_REGEX (0xadbeef00)
 #define REQUEST_MAX_REGEX (8)
@@ -108,9 +176,13 @@ struct auth_req {
 	uint32_t		magic; /* for debugging only */
 #endif
 	RADIUS_PACKET		*packet;
+#ifdef WITH_PROXY
 	RADIUS_PACKET		*proxy;
+#endif
 	RADIUS_PACKET		*reply;
+#ifdef WITH_PROXY
 	RADIUS_PACKET		*proxy_reply;
+#endif
 	VALUE_PAIR		*config_items;
 	VALUE_PAIR		*username;
 	VALUE_PAIR		*password;
@@ -119,16 +191,23 @@ struct auth_req {
 
 	request_data_t		*data;
 	RADCLIENT		*client;
+#ifdef HAVE_PTHREAD_H
 	child_pid_t    		child_pid;
+#endif
 	time_t			timestamp;
 	int			number; /* internal server number */
 
 	rad_listen_t		*listener;
+#ifdef WITH_PROXY
 	rad_listen_t		*proxy_listener;
+#endif
 
-	int                     simul_max;
+
+	int                     simul_max; /* see modcall.c && xlat.c */
+#ifdef WITH_SESSION_MGMT
 	int                     simul_count;
 	int                     simul_mpp; /* WEIRD: 1 is false, 2 is true */
+#endif
 
 	int			options; /* miscellanous options */
 	const char		*module; /* for debugging unresponsive children */
@@ -147,6 +226,7 @@ struct auth_req {
 	fr_event_callback_t	next_callback;
 
 	int			in_request_hash;
+#ifdef WITH_PROXY
 	int			in_proxy_hash;
 
 	home_server	       	*home_server;
@@ -156,12 +236,18 @@ struct auth_req {
 
 	int			num_proxied_requests;
 	int			num_proxied_responses;
+#endif
 
 	const char		*server;
 	REQUEST			*parent;
+	radlog_func_t		radlog;	/* logging function, if set */
 };				/* REQUEST typedef */
 
 #define RAD_REQUEST_OPTION_NONE            (0)
+#define RAD_REQUEST_OPTION_DEBUG           (1)
+#define RAD_REQUEST_OPTION_DEBUG2          (2)
+#define RAD_REQUEST_OPTION_DEBUG3          (3)
+#define RAD_REQUEST_OPTION_DEBUG4          (4)
 
 #define REQUEST_ACTIVE 		(1)
 #define REQUEST_STOP_PROCESSING (2)
@@ -207,6 +293,7 @@ struct rad_listen_t {
 	RAD_LISTEN_TYPE	type;
 	int		fd;
 	const char	*server;
+	int		status;
 
 	rad_listen_recv_t recv;
 	rad_listen_send_t send;
@@ -215,8 +302,16 @@ struct rad_listen_t {
 	rad_listen_print_t print;
 
 	void		*data;
+
+#ifdef WITH_STATS
+	fr_stats_t	stats;
+#endif
 };
 
+#define RAD_LISTEN_STATUS_INIT   (0)
+#define RAD_LISTEN_STATUS_KNOWN  (1)
+#define RAD_LISTEN_STATUS_CLOSED (2)
+#define RAD_LISTEN_STATUS_FINISH (3)
 
 typedef enum radlog_dest_t {
   RADLOG_STDOUT = 0,
@@ -255,13 +350,24 @@ typedef struct main_config_t {
 	radlog_dest_t	radlog_dest;
 	CONF_SECTION	*config;
 	const char	*name;
-	int		do_snmp;
 } MAIN_CONFIG_T;
 
 #define DEBUG	if(debug_flag)log_debug
 #define DEBUG2  if (debug_flag > 1)log_debug
 #define DEBUG3  if (debug_flag > 2)log_debug
 
+#if __GNUC__ >= 3
+#define RDEBUG(fmt, ...)   if(request && request->radlog) request->radlog(L_DBG, 1, request, fmt, ## __VA_ARGS__)
+#define RDEBUG2(fmt, ...)  if(request && request->radlog) request->radlog(L_DBG, 2, request, fmt, ## __VA_ARGS__)
+#define RDEBUG3(fmt, ...)  if(request && request->radlog) request->radlog(L_DBG, 3, request, fmt, ## __VA_ARGS__)
+#define RDEBUG4(fmt, ...)  if(request && request->radlog) request->radlog(L_DBG, 4, request, fmt, ## __VA_ARGS__)
+#else
+#define RDEBUG  DEBUG
+#define RDEBUG2 DEBUG2
+#define RDEBUG3 DEBUG3
+#define RDEBUG4 DEBUG4
+#endif
+
 #define SECONDS_PER_DAY		86400
 #define MAX_REQUEST_TIME	30
 #define CLEANUP_DELAY		5
@@ -379,11 +485,19 @@ void		clients_free(RADCLIENT_LIST *clients);
 RADCLIENT_LIST	*clients_parse_section(CONF_SECTION *section);
 void		client_free(RADCLIENT *client);
 int		client_add(RADCLIENT_LIST *clients, RADCLIENT *client);
+#ifdef WITH_DYNAMIC_CLIENTS
+void		client_delete(RADCLIENT_LIST *clients, RADCLIENT *client);
+RADCLIENT	*client_create(RADCLIENT_LIST *clients, REQUEST *request);
+#endif
 RADCLIENT	*client_find(const RADCLIENT_LIST *clients,
 			     const fr_ipaddr_t *ipaddr);
 RADCLIENT	*client_findbynumber(const RADCLIENT_LIST *clients,
 				     int number);
 RADCLIENT	*client_find_old(const fr_ipaddr_t *ipaddr);
+int		client_validate(RADCLIENT_LIST *clients, RADCLIENT *master,
+				RADCLIENT *c);
+RADCLIENT	*client_read(const char *filename, int in_server);
+
 
 /* files.c */
 int		pairlist_read(const char *file, PAIR_LIST **list, int complain);
@@ -405,6 +519,11 @@ int		log_debug(const char *, ...)
 #endif
 ;
 void 		vp_listdebug(VALUE_PAIR *vp);
+void radlog_request(int lvl, int priority, REQUEST *request, const char *msg, ...)
+#ifdef __GNUC__
+		__attribute__ ((format (printf, 4, 5)))
+#endif
+;
 
 /* auth.c */
 char	*auth_name(char *buf, size_t buflen, REQUEST *request, int do_cli);
@@ -433,6 +552,7 @@ int radius_compare_vps(REQUEST *request, VALUE_PAIR *check, VALUE_PAIR *vp);
 int radius_callback_compare(REQUEST *req, VALUE_PAIR *request,
 			    VALUE_PAIR *check, VALUE_PAIR *check_pairs,
 			    VALUE_PAIR **reply_pairs);
+int radius_find_compare(int attribute);
 VALUE_PAIR	*radius_paircreate(REQUEST *request, VALUE_PAIR **vps,
 				  int attribute, int type);
 VALUE_PAIR *radius_pairmake(REQUEST *request, VALUE_PAIR **vps,
@@ -458,6 +578,7 @@ extern		pid_t rad_waitpid(pid_t pid, int *status);
 extern          int total_active_threads(void);
 extern          void thread_pool_lock(void);
 extern          void thread_pool_unlock(void);
+extern		void thread_pool_queue_stats(int *array);
 
 #ifndef HAVE_PTHREAD_H
 #define rad_fork(n) fork()
@@ -477,6 +598,11 @@ int listen_init(CONF_SECTION *cs, rad_listen_t **head);
 rad_listen_t *proxy_new_listener(void);
 RADCLIENT *client_listener_find(const rad_listen_t *listener,
 				const fr_ipaddr_t *ipaddr);
+#ifdef WITH_STATS
+RADCLIENT_LIST *listener_find_client_list(const fr_ipaddr_t *ipaddr,
+					  int port);
+rad_listen_t *listener_find_byipaddr(const fr_ipaddr_t *ipaddr, int port);
+#endif
 
 /* event.c */
 int radius_event_init(CONF_SECTION *cs, int spawn_flag);
@@ -487,6 +613,7 @@ int received_request(rad_listen_t *listener,
 		     RADIUS_PACKET *packet, REQUEST **prequest,
 		     RADCLIENT *client);
 REQUEST *received_proxy_response(RADIUS_PACKET *packet);
+void event_new_fd(rad_listen_t *listener);
 
 /* evaluate.c */
 int radius_evaluate_condition(REQUEST *request, int modreturn, int depth,
