@@ -5,12 +5,12 @@
  * realms.h	Structures, prototypes and global variables
  *		for realms
  *
- * Version:	$Id: realms.h,v 1.16 2007/11/23 13:46:56 aland Exp $
+ * Version:	$Id$
  *
  */
 
 #include <freeradius-devel/ident.h>
-RCSIDH(realms_h, "$Id: realms.h,v 1.16 2007/11/23 13:46:56 aland Exp $")
+RCSIDH(realms_h, "$Id$")
 
 #define HOME_TYPE_INVALID (0)
 #define HOME_TYPE_AUTH    (1)
@@ -28,10 +28,10 @@ typedef struct home_server {
 	const char	*name;
 
 	const char	*hostname;
+	const char	*server; /* for internal proxying */
 
 	fr_ipaddr_t	ipaddr;
 
-
 	int		port;
 	int		type;		/* auth/acct */
 
@@ -47,8 +47,8 @@ typedef struct home_server {
 	int		response_window;
 	int		max_outstanding; /* don't overload it */
 	int		currently_outstanding;
-	uint32_t       	total_requests_sent;
 
+	struct timeval	revive_time;
 	struct timeval	zombie_period_start;
 	int		zombie_period; /* unresponsive for T, mark it dead */
 
@@ -65,6 +65,11 @@ typedef struct home_server {
 
 	int		revive_interval; /* if it doesn't support pings */
 	CONF_SECTION	*cs;
+#ifdef WITH_STATS
+	fr_stats_t	stats;
+
+	fr_stats_ema_t  ema;
+#endif
 } home_server;
 
 
@@ -85,7 +90,9 @@ typedef struct home_pool_t {
 	int			server_type;
 	CONF_SECTION		*cs;
 
-	const char		*virtual_server;
+	const char		*virtual_server; /* for pre/post-proxy */
+	
+	home_server		*fallback;
 
 	int			num_home_servers;
 	home_server		*servers[1];
@@ -103,7 +110,8 @@ typedef struct _realm {
 
 int realms_init(CONF_SECTION *config);
 void realms_free(void);
-REALM *realm_find(const char *name);
+REALM *realm_find(const char *name); /* name is from a packet */
+REALM *realm_find2(const char *name); /* ... with name taken from realm_find */
 
 home_server *home_server_ldb(const char *realmname, home_pool_t *pool, REQUEST *request);
 home_server *home_server_find(fr_ipaddr_t *ipaddr, int port);
