@@ -1,7 +1,7 @@
 /*
  * dhcp.c	Functions to send/receive dhcp packets.
  *
- * Version:	$Id: dhcp.c,v 1.2 2008/05/28 17:32:26 aland Exp $
+ * Version:	$Id$
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  */
 
 #include	<freeradius-devel/ident.h>
-RCSID("$Id: dhcp.c,v 1.2 2008/05/28 17:32:26 aland Exp $")
+RCSID("$Id$")
 
 #include <freeradius-devel/libradius.h>
 #include <freeradius-devel/udpfromto.h>
@@ -106,7 +106,7 @@ static int dhcp_header_sizes[] = {
  *	Some clients silently ignore responses less than 300 bytes.
  */
 #define MIN_PACKET_SIZE (244)
-#define DEFAULT_PACKET_SIZE (576)
+#define DEFAULT_PACKET_SIZE (300)
 #define MAX_PACKET_SIZE (1500 - 40)
 
 /*
@@ -118,12 +118,10 @@ RADIUS_PACKET *fr_dhcp_recv(int sockfd)
 	uint32_t		magic;
 	struct sockaddr_storage	src;
 	struct sockaddr_storage	dst;
-	socklen_t		sizeof_src = sizeof(src);
-	socklen_t	        sizeof_dst = sizeof(dst);
+	socklen_t		sizeof_src;
+	socklen_t	        sizeof_dst;
 	RADIUS_PACKET		*packet;
-
-	memset(&src, 0, sizeof_src);
-	memset(&dst, 0, sizeof_dst);
+	int port;
 
 	packet = rad_alloc(0);
 	if (!packet) return NULL;
@@ -136,6 +134,7 @@ RADIUS_PACKET *fr_dhcp_recv(int sockfd)
 	}
 
 	packet->sockfd = sockfd;
+	sizeof_src = sizeof(src);
 	packet->data_len = recvfrom(sockfd, packet->data, MAX_PACKET_SIZE, 0,
 				    (struct sockaddr *)&src, &sizeof_src);
 	if (packet->data_len <= 0) {
@@ -191,7 +190,7 @@ RADIUS_PACKET *fr_dhcp_recv(int sockfd)
 	if ((packet->data[240] != 53) ||
 	    (packet->data[241] != 1) ||
 	    (packet->data[242] == 0) ||
-	    (packet->data[242] >= 8)) {
+	    (packet->data[242] > 8)) {
 		fprintf(stderr, "Unknown, or badly formatted DHCP packet\n");
 		rad_free(&packet);
 		return NULL;
@@ -227,28 +226,19 @@ RADIUS_PACKET *fr_dhcp_recv(int sockfd)
 	 *	FIXME: More checks, like DHCP packet type?
 	 */
 
-	{
-		struct sockaddr_in	*s4;
-		struct sockaddr_storage si;
-		socklen_t si_len = sizeof(si);
+	sizeof_dst = sizeof(dst);
+	/*
+	 *	This should never fail...
+	 */
+	getsockname(sockfd, (struct sockaddr *) &dst, &sizeof_dst);
+	
+	fr_sockaddr2ipaddr(&src, sizeof_src, &packet->src_ipaddr, &port);
+	packet->src_port = port;
 
-		/*
-		 *	This should never fail...
-		 */
-		getsockname(sockfd, (struct sockaddr *) &si, &si_len);
+	fr_sockaddr2ipaddr(&dst, sizeof_dst, &packet->dst_ipaddr, &port);
+	packet->dst_port = port;
 
-		s4 = (struct sockaddr_in *)&src;
-		packet->src_ipaddr.af = AF_INET;
-		packet->src_ipaddr.ipaddr.ip4addr = s4->sin_addr;
-		packet->src_port = ntohs(s4->sin_port);
-
-		s4 = (struct sockaddr_in *)&si;
-		packet->dst_ipaddr.af = AF_INET;
-		packet->dst_ipaddr.ipaddr.ip4addr = s4->sin_addr;
-		packet->dst_port = ntohs(s4->sin_port);
-	}
-
-	if (librad_debug > 1) {
+	if (fr_debug_flag > 1) {
 		char type_buf[64];
 		const char *name = type_buf;
 		char src_ip_buf[256], dst_ip_buf[256];
@@ -285,32 +275,17 @@ int fr_dhcp_send(RADIUS_PACKET *packet)
 {
 	struct sockaddr_storage	dst;
 	struct sockaddr_storage	src;
-	socklen_t		sizeof_dst = sizeof(dst);
-	socklen_t		sizeof_src = sizeof(src);
+	socklen_t		sizeof_dst;
+	socklen_t		sizeof_src;
 
-	memset(&src, 0, sizeof(src));
-	memset(&dst, 0, sizeof(dst));
+	fr_ipaddr2sockaddr(&packet->dst_ipaddr, packet->dst_port,
+			   &dst, &sizeof_dst);
 
 	/*
-	 *	Only IPv4 is supported.
+	 *	Currently unused...
 	 */
-	{
-		struct sockaddr_in	*s4;
-
-		s4 = (struct sockaddr_in *)&dst;
-		sizeof_dst = sizeof(struct sockaddr_in);
-
-		s4->sin_family = AF_INET;
-		s4->sin_addr = packet->dst_ipaddr.ipaddr.ip4addr;
-		s4->sin_port = htons(packet->dst_port);
-
-		s4 = (struct sockaddr_in *)&src;
-		sizeof_src = sizeof(struct sockaddr_in);
-
-		s4->sin_family = AF_INET;
-		s4->sin_addr = packet->src_ipaddr.ipaddr.ip4addr;
-		s4->sin_port = htons(packet->src_port);
-	}
+	fr_ipaddr2sockaddr(&packet->src_ipaddr, packet->src_port,
+			   &src, &sizeof_src);
 
 	/*
 	 *	Assume that the packet is encoded before sending it.
@@ -334,7 +309,7 @@ int fr_dhcp_decode(RADIUS_PACKET *packet)
 	tail = &head;
 	p = packet->data;
 	
-	if ((librad_debug > 2) && fr_log_fp) {
+	if ((fr_debug_flag > 2) && fr_log_fp) {
 		for (i = 0; i < packet->data_len; i++) {
 			if ((i & 0x0f) == 0x00) fprintf(stderr, "%d: ", i);
 			fprintf(fr_log_fp, "%02x ", packet->data[i]);
@@ -355,7 +330,7 @@ int fr_dhcp_decode(RADIUS_PACKET *packet)
 	for (i = 0; i < 14; i++) {
 		vp = pairmake(dhcp_header_names[i], NULL, T_OP_EQ);
 		if (!vp) {
-			fprintf(stderr, "Parse error %s\n", librad_errstr);
+			fprintf(stderr, "Parse error %s\n", fr_strerror());
 			pairfree(&head);
 			return -1;
 		}
@@ -417,7 +392,7 @@ int fr_dhcp_decode(RADIUS_PACKET *packet)
 
 		if (!vp) continue;
 		
-		if (librad_debug > 1) {
+		if (fr_debug_flag > 1) {
 			vp_prints(buffer, sizeof(buffer), vp);
 			fprintf(stderr, "\t%s\n", buffer);
 		}
@@ -512,7 +487,7 @@ int fr_dhcp_decode(RADIUS_PACKET *packet)
 			vp = pairmake(da->name, NULL, T_OP_EQ);
 			if (!vp) {
 				fprintf(stderr, "Cannot build attribute %s\n",
-					librad_errstr);
+					fr_strerror());
 				pairfree(&head);
 				return -1;
 			}
@@ -554,7 +529,7 @@ int fr_dhcp_decode(RADIUS_PACKET *packet)
 				raw:
 					vp = pairmake(da->name, NULL, T_OP_EQ);
 					if (!vp) {
-						fprintf(stderr, "Cannot build attribute %s\n", librad_errstr);
+						fprintf(stderr, "Cannot build attribute %s\n", fr_strerror());
 						pairfree(&head);
 						return -1;
 					}
@@ -573,7 +548,7 @@ int fr_dhcp_decode(RADIUS_PACKET *packet)
 				
 			vp->length = alen;
 
-			if (librad_debug > 1) {
+			if (fr_debug_flag > 1) {
 				vp_prints(buffer, sizeof(buffer), vp);
 				fprintf(stderr, "\t%s\n", buffer);
 			}
@@ -646,7 +621,7 @@ int fr_dhcp_decode(RADIUS_PACKET *packet)
 		maxms->vp_integer = mtu->vp_integer;
 	}
 
-	if (librad_debug) fflush(stdout);
+	if (fr_debug_flag) fflush(stdout);
 
 	return 0;
 }
@@ -754,7 +729,15 @@ int fr_dhcp_encode(RADIUS_PACKET *packet, RADIUS_PACKET *original)
 
 	if (packet->code == 0) packet->code = PW_DHCP_NAK;
 
-	if (librad_debug > 1) {
+	/*
+	 *	FIXME: allow it to send client packets.
+	 */
+	if (!original) {
+		fr_strerror_printf("Need original to send response!");
+		return -1;
+	}
+
+	if (fr_debug_flag > 1) {
 		char type_buf[64];
 		const char *name = type_buf;
 		char src_ip_buf[256], dst_ip_buf[256];
@@ -863,6 +846,16 @@ int fr_dhcp_encode(RADIUS_PACKET *packet, RADIUS_PACKET *original)
 	p += 2;
 
 	memcpy(p, original->data + 10, 6); /* copy flags && ciaddr */
+
+	/*
+	 *	Allow the admin to set the broadcast flag.
+	 */
+	vp = pairfind(packet->vps, DHCP2ATTR(262));
+	if (vp) {
+		p[0] |= (vp->vp_integer & 0xff00) >> 8;
+		p[1] |= (vp->vp_integer & 0xff);
+	}
+
 	p += 6;
 
 	/*
@@ -880,17 +873,9 @@ int fr_dhcp_encode(RADIUS_PACKET *packet, RADIUS_PACKET *original)
 	memset(p, 0, 4);	/* siaddr is zero */
 	p += 4;
 
-	memset(p, 0, 4);	/* gateway address is zero */
+	memcpy(p, original->data + 24, 4); /* copy gateway IP address */
 	p += 4;
 
-	/*
-	 *	FIXME: allow it to send client packets.
-	 */
-	if (!original) {
-		librad_log("Need original to send response!");
-		return -1;
-	}
-
 	memcpy(p, original->data + 28, DHCP_CHADDR_LEN);
 	p += DHCP_CHADDR_LEN;
 
@@ -904,7 +889,7 @@ int fr_dhcp_encode(RADIUS_PACKET *packet, RADIUS_PACKET *original)
 	/*
 	 *	Print the header.
 	 */
-	if (librad_debug > 1) {
+	if (fr_debug_flag > 1) {
 		uint8_t *pp = p;
 
 		p = packet->data;
@@ -912,7 +897,7 @@ int fr_dhcp_encode(RADIUS_PACKET *packet, RADIUS_PACKET *original)
 		for (i = 0; i < 14; i++) {
 			vp = pairmake(dhcp_header_names[i], NULL, T_OP_EQ);
 			if (!vp) {
-				fprintf(stderr, "Parse error %s\n", librad_errstr);
+				fprintf(stderr, "Parse error %s\n", fr_strerror());
 				return -1;
 			}
 			
@@ -973,13 +958,6 @@ int fr_dhcp_encode(RADIUS_PACKET *packet, RADIUS_PACKET *original)
 		p = pp;
 	}
 
-	vp = pairfind(packet->vps, DHCP2ATTR(53));
-	if (vp && (vp->vp_integer != (packet->code - PW_DHCP_OFFSET))) {
-		fprintf(stderr, "Message-Type doesn't match! %d %d\n",
-			packet->code, vp->vp_integer);
-	}
-	pairdelete(&packet->vps, DHCP2ATTR(0x35));
-
 	/*
 	 *	Before packing the attributes, re-order them so that
 	 *	the array ones are all contiguous.  This simplifies
@@ -1063,7 +1041,7 @@ int fr_dhcp_encode(RADIUS_PACKET *packet, RADIUS_PACKET *original)
 		}
 
 		for (i = 0; i < num_entries; i++) {
-			if (librad_debug > 1) {
+			if (fr_debug_flag > 1) {
 				vp_prints(buffer, sizeof(buffer), vp);
 				fprintf(stderr, "\t%s\n", buffer);
 			}
@@ -1155,6 +1133,12 @@ int fr_dhcp_encode(RADIUS_PACKET *packet, RADIUS_PACKET *original)
 	if (dhcp->giaddr != htonl(INADDR_ANY)) {
 		packet->dst_ipaddr.ipaddr.ip4addr.s_addr = dhcp->giaddr;
 
+		/*
+		 *	Gateways send FROM 68 to 67, and we
+		 *	respond FROM 67 to 67... not to 68.
+		 */
+		packet->dst_port = original->dst_port;
+
 	} else if (packet->code == PW_DHCP_NAK) {
 		packet->dst_ipaddr.ipaddr.ip4addr.s_addr = htonl(INADDR_BROADCAST);
 		
@@ -1186,7 +1170,7 @@ int fr_dhcp_encode(RADIUS_PACKET *packet, RADIUS_PACKET *original)
 		packet->data_len = DEFAULT_PACKET_SIZE;
 	}
 
-	if ((librad_debug > 2) && fr_log_fp) {
+	if ((fr_debug_flag > 2) && fr_log_fp) {
 		for (i = 0; i < packet->data_len; i++) {
 			if ((i & 0x0f) == 0x00) fprintf(fr_log_fp, "%d: ", i);
 			fprintf(fr_log_fp, "%02x ", packet->data[i]);
