@@ -1,7 +1,7 @@
 /*
  * vqp.c	Functions to send/receive VQP packets.
  *
- * Version:	$Id: vqp.c,v 1.4 2008/01/05 17:58:44 nbk Exp $
+ * Version:	$Id$
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Lesser General Public
@@ -21,7 +21,7 @@
  */
 
 #include	<freeradius-devel/ident.h>
-RCSID("$Id: vqp.c,v 1.4 2008/01/05 17:58:44 nbk Exp $");
+RCSID("$Id$");
 
 #include	<freeradius-devel/libradius.h>
 #include	<freeradius-devel/udpfromto.h>
@@ -78,63 +78,20 @@ static int vqp_sendto(int sockfd, void *data, size_t data_len, int flags,
 		      int dst_port)
 {
 	struct sockaddr_storage	dst;
-	socklen_t		sizeof_dst = sizeof(dst);
+	socklen_t		sizeof_dst;
 
 #ifdef WITH_UDPFROMTO
 	struct sockaddr_storage	src;
-	socklen_t		sizeof_src = sizeof(src);
+	socklen_t		sizeof_src;
 
-	memset(&src, 0, sizeof(src));
-#endif
-	memset(&dst, 0, sizeof(dst));
-
-	/*
-	 *	IPv4 is supported.
-	 */
-	if (dst_ipaddr->af == AF_INET) {
-		struct sockaddr_in	*s4;
-
-		s4 = (struct sockaddr_in *)&dst;
-		sizeof_dst = sizeof(struct sockaddr_in);
-
-		s4->sin_family = AF_INET;
-		s4->sin_addr = dst_ipaddr->ipaddr.ip4addr;
-		s4->sin_port = htons(dst_port);
-
-#ifdef WITH_UDPFROMTO
-		s4 = (struct sockaddr_in *)&src;
-		sizeof_src = sizeof(struct sockaddr_in);
-
-		s4->sin_family = AF_INET;
-		s4->sin_addr = src_ipaddr->ipaddr.ip4addr;
+	fr_ipaddr2sockaddr(src_ipaddr, 0, &src, &sizeof_src);
+#else
+	src_ipaddr = src_ipaddr; /* -Wunused */
 #endif
 
-	/*
-	 *	IPv6 MAY be supported.
-	 */
-#ifdef HAVE_STRUCT_SOCKADDR_IN6
-	} else if (dst_ipaddr->af == AF_INET6) {
-		struct sockaddr_in6	*s6;
-
-		s6 = (struct sockaddr_in6 *)&dst;
-		sizeof_dst = sizeof(struct sockaddr_in6);
-		
-		s6->sin6_family = AF_INET6;
-		s6->sin6_addr = dst_ipaddr->ipaddr.ip6addr;
-		s6->sin6_port = htons(dst_port);
-
-#ifdef WITH_UDPFROMTO
-		return -1;	/* UDPFROMTO && IPv6 are not supported */
-#if 0
-		s6 = (struct sockaddr_in6 *)&src;
-		sizeof_src = sizeof(struct sockaddr_in6);
-
-		s6->sin6_family = AF_INET6;
-		s6->sin6_addr = src_ipaddr->ipaddr.ip6addr;
-#endif /* #if 0 */
-#endif /* WITH_UDPFROMTO */
-#endif /* HAVE_STRUCT_SOCKADDR_IN6 */
-	} else return -1;   /* Unknown address family, Die Die Die! */
+	if (!fr_ipaddr2sockaddr(dst_ipaddr, dst_port, &dst, &sizeof_dst)) {
+		return -1;   /* Unknown address family, Die Die Die! */
+	}
 
 #ifdef WITH_UDPFROMTO
 	/*
@@ -178,6 +135,7 @@ static ssize_t vqp_recvfrom(int sockfd, uint8_t **pbuf, int flags,
 	uint8_t			header[4];
 	void			*buf;
 	size_t			len;
+	int			port;
 
 	memset(&src, 0, sizeof_src);
 	memset(&dst, 0, sizeof_dst);
@@ -285,41 +243,15 @@ static ssize_t vqp_recvfrom(int sockfd, uint8_t **pbuf, int flags,
 		return data_len;
 	}
 
-	/*
-	 *	Check address families, and update src/dst ports, etc.
-	 */
-	if (src.ss_family == AF_INET) {
-		struct sockaddr_in	*s4;
-
-		s4 = (struct sockaddr_in *)&src;
-		src_ipaddr->af = AF_INET;
-		src_ipaddr->ipaddr.ip4addr = s4->sin_addr;
-		*src_port = ntohs(s4->sin_port);
-
-		s4 = (struct sockaddr_in *)&dst;
-		dst_ipaddr->af = AF_INET;
-		dst_ipaddr->ipaddr.ip4addr = s4->sin_addr;
-		*dst_port = ntohs(s4->sin_port);
-
-#ifdef HAVE_STRUCT_SOCKADDR_IN6
-	} else if (src.ss_family == AF_INET6) {
-		struct sockaddr_in6	*s6;
-
-		s6 = (struct sockaddr_in6 *)&src;
-		src_ipaddr->af = AF_INET6;
-		src_ipaddr->ipaddr.ip6addr = s6->sin6_addr;
-		*src_port = ntohs(s6->sin6_port);
-
-		s6 = (struct sockaddr_in6 *)&dst;
-		dst_ipaddr->af = AF_INET6;
-		dst_ipaddr->ipaddr.ip6addr = s6->sin6_addr;
-		*dst_port = ntohs(s6->sin6_port);
-#endif
-	} else {
+	if (!fr_sockaddr2ipaddr(&src, sizeof_src, src_ipaddr, &port)) {
 		free(buf);
 		return -1;	/* Unknown address family, Die Die Die! */
 	}
-	
+	*src_port = port;
+
+	fr_sockaddr2ipaddr(&dst, sizeof_dst, dst_ipaddr, &port);
+	*dst_port = port;
+
 	/*
 	 *	Different address families should never happen.
 	 */
@@ -347,7 +279,7 @@ RADIUS_PACKET *vqp_recv(int sockfd)
 	 *	Allocate the new request data structure
 	 */
 	if ((packet = malloc(sizeof(*packet))) == NULL) {
-		librad_log("out of memory");
+		fr_strerror_printf("out of memory");
 		return NULL;
 	}
 	memset(packet, 0, sizeof(*packet));
@@ -360,7 +292,7 @@ RADIUS_PACKET *vqp_recv(int sockfd)
 	 *	Check for socket errors.
 	 */
 	if (packet->data_len < 0) {
-		librad_log("Error receiving packet: %s", strerror(errno));
+		fr_strerror_printf("Error receiving packet: %s", strerror(errno));
 		/* packet->data is NULL */
 		free(packet);
 		return NULL;
@@ -373,7 +305,7 @@ RADIUS_PACKET *vqp_recv(int sockfd)
 	 *	packet than normal implementations may send.
 	 */
 	if (packet->data_len < VQP_HDR_LEN) {
-		librad_log("VQP packet is too short");
+		fr_strerror_printf("VQP packet is too short");
 		rad_free(&packet);
 		return NULL;
 	}
@@ -391,7 +323,7 @@ RADIUS_PACKET *vqp_recv(int sockfd)
 	}
 
 	if (ptr[3] > VQP_MAX_ATTRIBUTES) {
-		librad_log("Too many VQP attributes");
+		fr_strerror_printf("Too many VQP attributes");
 		rad_free(&packet);
 		return NULL;
 	}
@@ -407,7 +339,7 @@ RADIUS_PACKET *vqp_recv(int sockfd)
 
 		while (length > 0) {
 			if (length < 7) {
-				librad_log("Packet contains malformed attribute");
+				fr_strerror_printf("Packet contains malformed attribute");
 				rad_free(&packet);
 				return NULL;
 			}
@@ -418,7 +350,7 @@ RADIUS_PACKET *vqp_recv(int sockfd)
 			 */
 			if ((ptr[0] != 0) || (ptr[1] != 0) ||
 			    (ptr[2] != 0x0c) || (ptr[3] < 1) || (ptr[3] > 8)) {
-				librad_log("Packet contains invalid attribute");
+				fr_strerror_printf("Packet contains invalid attribute");
 				rad_free(&packet);
 				return NULL;
 			}
@@ -432,7 +364,7 @@ RADIUS_PACKET *vqp_recv(int sockfd)
 			 *	won't be typing in a 32K vlan name.
 			 */
 			if ((ptr[4] != 0) || (ptr[5] > 253)) {
-				librad_log("Packet contains attribute with invalid length %02x %02x", ptr[4], ptr[5]);
+				fr_strerror_printf("Packet contains attribute with invalid length %02x %02x", ptr[4], ptr[5]);
 				rad_free(&packet);
 				return NULL;
 			}
@@ -499,7 +431,7 @@ int vqp_decode(RADIUS_PACKET *packet)
 
 	vp = paircreate(PW_VQP_PACKET_TYPE, PW_TYPE_OCTETS);
 	if (!vp) {
-		librad_log("No memory");
+		fr_strerror_printf("No memory");
 		return -1;
 	}
 	vp->lvalue = packet->data[1];
@@ -510,7 +442,7 @@ int vqp_decode(RADIUS_PACKET *packet)
 
 	vp = paircreate(PW_VQP_ERROR_CODE, PW_TYPE_OCTETS);
 	if (!vp) {
-		librad_log("No memory");
+		fr_strerror_printf("No memory");
 		return -1;
 	}
 	vp->lvalue = packet->data[2];
@@ -521,7 +453,7 @@ int vqp_decode(RADIUS_PACKET *packet)
 
 	vp = paircreate(PW_VQP_SEQUENCE_NUMBER, PW_TYPE_OCTETS);
 	if (!vp) {
-		librad_log("No memory");
+		fr_strerror_printf("No memory");
 		return -1;
 	}
 	vp->lvalue = packet->id; /* already set by vqp_recv */
@@ -551,7 +483,7 @@ int vqp_decode(RADIUS_PACKET *packet)
 		if (!vp) {
 			pairfree(&packet->vps);
 
-			librad_log("No memory");
+			fr_strerror_printf("No memory");
 			return -1;
 		}
 
@@ -611,7 +543,7 @@ int vqp_encode(RADIUS_PACKET *packet, RADIUS_PACKET *original)
 	VALUE_PAIR	*vps[VQP_MAX_ATTRIBUTES];
 
 	if (!packet) {
-		librad_log("Failed encoding VQP");
+		fr_strerror_printf("Failed encoding VQP");
 		return -1;
 	}
 
@@ -619,13 +551,13 @@ int vqp_encode(RADIUS_PACKET *packet, RADIUS_PACKET *original)
 
 	vp = pairfind(packet->vps, PW_VQP_PACKET_TYPE);
 	if (!vp) {
-		librad_log("Failed to find VQP-Packet-Type in response packet");
+		fr_strerror_printf("Failed to find VQP-Packet-Type in response packet");
 		return -1;
 	}
 
 	code = vp->lvalue;
 	if ((code < 1) || (code > 4)) {
-		librad_log("Invalid value %d for VQP-Packet-Type", code);
+		fr_strerror_printf("Invalid value %d for VQP-Packet-Type", code);
 		return -1;
 	}
 
@@ -653,7 +585,7 @@ int vqp_encode(RADIUS_PACKET *packet, RADIUS_PACKET *original)
 		 *	FIXME: Print the name...
 		 */
 		if (!vps[i]) {
-			librad_log("Failed to find VQP attribute %02x",
+			fr_strerror_printf("Failed to find VQP attribute %02x",
 				   contents[code][i]);
 			return -1;
 		}
@@ -664,7 +596,7 @@ int vqp_encode(RADIUS_PACKET *packet, RADIUS_PACKET *original)
 
 	packet->data = malloc(length);
 	if (!packet->data) {
-		librad_log("No memory");
+		fr_strerror_printf("No memory");
 		return -1;
 	}
 	packet->data_len = length;
@@ -693,7 +625,7 @@ int vqp_encode(RADIUS_PACKET *packet, RADIUS_PACKET *original)
 		memcpy(ptr + 4, &sequence, 4);
 	} else {
 		if (!original) {
-			librad_log("Cannot send VQP response without request");
+			fr_strerror_printf("Cannot send VQP response without request");
 			return -1;
 		}
 
