@@ -1,7 +1,7 @@
 /*
  * client.c	Read clients into memory.
  *
- * Version:     $Id: client.c,v 1.82 2008/06/06 12:56:22 aland Exp $
+ * Version:     $Id$
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,10 +23,9 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: client.c,v 1.82 2008/06/06 12:56:22 aland Exp $")
+RCSID("$Id$")
 
 #include <freeradius-devel/radiusd.h>
-#include <freeradius-devel/radius_snmp.h>
 #include <freeradius-devel/rad_assert.h>
 
 #include <sys/stat.h>
@@ -34,6 +33,12 @@ RCSID("$Id: client.c,v 1.82 2008/06/06 12:56:22 aland Exp $")
 #include <ctype.h>
 #include <fcntl.h>
 
+#ifdef WITH_DYNAMIC_CLIENTS
+#ifdef HAVE_DIRENT_H
+#include <dirent.h>
+#endif
+#endif
+
 struct radclient_list {
 	/*
 	 *	FIXME: One set of trees for IPv4, and another for IPv6?
@@ -43,7 +48,7 @@ struct radclient_list {
 };
 
 
-#ifdef WITH_SNMP
+#ifdef WITH_STATS
 static rbtree_t		*tree_num;	/* client numbers 0..N */
 static int		tree_num_max;
 #endif
@@ -62,17 +67,20 @@ void client_free(RADCLIENT *client)
 	free(client->password);
 	free(client->server);
 
-#ifdef WITH_SNMP
+#ifdef WITH_STATS
 	free(client->auth);
 #ifdef WITH_ACCOUNTING
 	free(client->acct);
 #endif
 #endif
 
+#ifdef WITH_DYNAMIC_CLIENTS
+	free(client->client_server);
+#endif
+
 	free(client);
 }
 
-
 /*
  *	Callback for comparing two clients.
  */
@@ -84,7 +92,7 @@ static int client_ipaddr_cmp(const void *one, const void *two)
 	return fr_ipaddr_cmp(&a->ipaddr, &b->ipaddr);
 }
 
-#ifdef WITH_SNMP
+#ifdef WITH_STATS
 static int client_num_cmp(const void *one, const void *two)
 {
 	const RADCLIENT *a = one;
@@ -109,7 +117,7 @@ void clients_free(RADCLIENT_LIST *clients)
 	}
 
 	if (clients == root_clients) {
-#ifdef WITH_SNMP
+#ifdef WITH_STATS
 		if (tree_num) rbtree_free(tree_num);
 		tree_num = NULL;
 		tree_num_max = 0;
@@ -246,7 +254,7 @@ int client_add(RADCLIENT_LIST *clients, RADCLIENT *client)
 		return 0;
 	}
 
-#ifdef WITH_SNMP
+#ifdef WITH_STATS
 	if (!tree_num) {
 		tree_num = rbtree_create(client_num_cmp, NULL, 0);
 	}
@@ -267,6 +275,28 @@ int client_add(RADCLIENT_LIST *clients, RADCLIENT *client)
 	}
 #endif
 
+#ifdef WITH_DYNAMIC_CLIENTS
+	/*
+	 *	More catching of clients added by rlm_sql.
+	 *
+	 *	The sql modules sets the dynamic flag BEFORE calling
+	 *	us.  The client_create() function sets it AFTER
+	 *	calling us.
+	 */
+	if (client->dynamic && (client->lifetime == 0)) {
+		RADCLIENT *network;
+
+		/*
+		 *	If there IS an enclosing network,
+		 *	inherit the lifetime from it.
+		 */
+		network = client_find(clients, &client->ipaddr);
+		if (network) {
+			client->lifetime = network->lifetime;
+		}
+	}
+#endif
+
 	client->number = tree_num_max;
 	tree_num_max++;
 	if (tree_num) rbtree_insert(tree_num, client);
@@ -280,14 +310,26 @@ int client_add(RADCLIENT_LIST *clients, RADCLIENT *client)
 }
 
 
+#ifdef WITH_DYNAMIC_CLIENTS
+void client_delete(RADCLIENT_LIST *clients, RADCLIENT *client)
+{
+	if (!clients || !client) return;
+
+	rad_assert((client->prefix >= 0) && (client->prefix <= 128));
+
+	rbtree_deletebydata(clients->trees[client->prefix], client);
+}
+#endif
+
+
 /*
  *	Find a client in the RADCLIENTS list by number.
- *	This is a support function for the SNMP code.
+ *	This is a support function for the statistics code.
  */
 RADCLIENT *client_findbynumber(const RADCLIENT_LIST *clients,
 			       int number)
 {
-#ifdef WITH_SNMP
+#ifdef WITH_STATS
 	if (!clients) clients = root_clients;
 
 	if (!clients) return NULL;
@@ -391,6 +433,13 @@ static const CONF_PARSER client_config[] = {
 	{ "server",  PW_TYPE_STRING_PTR, /* compatability with 2.0-pre */
 	  offsetof(RADCLIENT, server), 0, NULL },
 
+#ifdef WITH_DYNAMIC_CLIENTS
+	{ "dynamic_clients",  PW_TYPE_STRING_PTR,
+	  offsetof(RADCLIENT, client_server), 0, NULL },
+	{ "lifetime",  PW_TYPE_INTEGER,
+	  offsetof(RADCLIENT, lifetime), 0, NULL },
+#endif
+
 	{ NULL, -1, 0, NULL, NULL }
 };
 
@@ -414,7 +463,7 @@ static RADCLIENT *client_parse(CONF_SECTION *cs, int in_server)
 	memset(c, 0, sizeof(*c));
 	c->cs = cs;
 
-#ifdef WITH_SNMP
+#ifdef WITH_STATS
 	c->auth = rad_malloc(sizeof(*c->auth));
 	memset(c->auth, 0, sizeof(*c->auth));
 
@@ -479,7 +528,7 @@ static RADCLIENT *client_parse(CONF_SECTION *cs, int in_server)
 			client_free(c);
 			cf_log_err(cf_sectiontoitem(cs),
 				   "Failed to look up hostname %s: %s",
-				   name2, librad_errstr);
+				   name2, fr_strerror());
 			return NULL;
 		}
 
@@ -542,6 +591,25 @@ static RADCLIENT *client_parse(CONF_SECTION *cs, int in_server)
 		break;
 	}
 
+#ifdef WITH_DYNAMIC_CLIENTS
+	if (c->client_server) {
+		free(c->secret);
+		c->secret = strdup("testing123");
+
+		if (((c->ipaddr.af == AF_INET) &&
+		     (c->prefix == 32)) ||
+		    ((c->ipaddr.af == AF_INET6) &&
+		     (c->prefix == 128))) {
+			cf_log_err(cf_sectiontoitem(cs),
+				   "Dynamic clients MUST be a network, not a single IP address.");
+			client_free(c);
+			return NULL;
+		}
+
+		return c;
+	}
+#endif
+
 	if (!c->secret || !*c->secret) {
 #ifdef WITH_DHCP
 		const char *value = NULL;
@@ -565,6 +633,7 @@ static RADCLIENT *client_parse(CONF_SECTION *cs, int in_server)
 	return c;
 }
 
+
 /*
  *	Create the linked list of clients from the new configuration
  *	type.  This way we don't have to change too much in the other
@@ -616,6 +685,88 @@ RADCLIENT_LIST *clients_parse_section(CONF_SECTION *section)
 		 *	for migration issues.
 		 */
 
+#ifdef WITH_DYNAMIC_CLIENTS
+#ifdef HAVE_DIRENT_H
+		if (c->client_server) {
+			const char *value;
+			CONF_PAIR *cp;
+			DIR		*dir;
+			struct dirent	*dp;
+			struct stat stat_buf;
+			char buf2[2048];
+
+			/*
+			 *	Find the directory where individual
+			 *	client definitions are stored.
+			 */
+			cp = cf_pair_find(cs, "directory");
+			if (!cp) continue;
+			
+			value = cf_pair_value(cp);
+			if (!value) {
+				cf_log_err(cf_sectiontoitem(cs),
+					   "The \"directory\" entry must not be empty");
+				client_free(c);
+				return NULL;
+			}
+
+			DEBUG("including dynamic clients in %s", value);
+			
+			dir = opendir(value);
+			if (!dir) {
+				cf_log_err(cf_sectiontoitem(cs), "Error reading directory %s: %s", value, strerror(errno));
+				client_free(c);
+				return NULL;
+			}
+			
+			/*
+			 *	Read the directory, ignoring "." files.
+			 */
+			while ((dp = readdir(dir)) != NULL) {
+				const char *p;
+				RADCLIENT *dc;
+
+				if (dp->d_name[0] == '.') continue;
+
+				/*
+				 *	Check for valid characters
+				 */
+				for (p = dp->d_name; *p != '\0'; p++) {
+					if (isalpha((int)*p) ||
+					    isdigit((int)*p) ||
+					    (*p == ':') ||
+					    (*p == '.')) continue;
+						break;
+				}
+				if (*p != '\0') continue;
+
+				snprintf(buf2, sizeof(buf2), "%s/%s",
+					 value, dp->d_name);
+
+				if ((stat(buf2, &stat_buf) != 0) ||
+				    S_ISDIR(stat_buf.st_mode)) continue;
+
+				dc = client_read(buf2, in_server);
+				if (!dc) {
+					cf_log_err(cf_sectiontoitem(cs),
+						   "Failed reading client file \"%s\"", buf2);
+					client_free(c);
+					return NULL;
+				}
+
+				/*
+				 *	Validate, and add to the list.
+				 */
+				if (!client_validate(clients, c, dc)) {
+					
+					client_free(c);
+					return NULL;
+				}
+			} /* loop over the directory */
+		}
+#endif /* HAVE_DIRENT_H */
+#endif /* WITH_DYNAMIC_CLIENTS */
+
 		if (!client_add(clients, c)) {
 			cf_log_err(cf_sectiontoitem(cs),
 				   "Failed to add client %s",
@@ -623,6 +774,7 @@ RADCLIENT_LIST *clients_parse_section(CONF_SECTION *section)
 			client_free(c);
 			return NULL;
 		}
+
 	}
 
 	/*
@@ -636,3 +788,222 @@ RADCLIENT_LIST *clients_parse_section(CONF_SECTION *section)
 
 	return clients;
 }
+
+#ifdef WITH_DYNAMIC_CLIENTS
+/*
+ *	We overload this structure a lot.
+ */
+static const CONF_PARSER dynamic_config[] = {
+	{ "FreeRADIUS-Client-IP-Address",  PW_TYPE_IPADDR,
+	  offsetof(RADCLIENT, ipaddr), 0, NULL },
+	{ "FreeRADIUS-Client-IPv6-Address",  PW_TYPE_IPV6ADDR,
+	  offsetof(RADCLIENT, ipaddr), 0, NULL },
+
+	{ "FreeRADIUS-Client-Require-MA",  PW_TYPE_BOOLEAN,
+	  offsetof(RADCLIENT, message_authenticator), NULL, NULL },
+
+	{ "FreeRADIUS-Client-Secret",  PW_TYPE_STRING_PTR,
+	  offsetof(RADCLIENT, secret), 0, "" },
+	{ "FreeRADIUS-Client-Shortname",  PW_TYPE_STRING_PTR,
+	  offsetof(RADCLIENT, shortname), 0, "" },
+	{ "FreeRADIUS-Client-NAS-Type",  PW_TYPE_STRING_PTR,
+	  offsetof(RADCLIENT, nastype), 0, NULL },
+	{ "FreeRADIUS-Client-Virtual-Server",  PW_TYPE_STRING_PTR,
+	  offsetof(RADCLIENT, server), 0, NULL },
+
+	{ NULL, -1, 0, NULL, NULL }
+};
+
+
+int client_validate(RADCLIENT_LIST *clients, RADCLIENT *master, RADCLIENT *c)
+{
+	char buffer[128];
+
+	/*
+	 *	No virtual server defined.  Inherit the parent's
+	 *	definition.
+	 */
+	if (master->server && !c->server) {
+		c->server = strdup(master->server);
+	}
+
+	/*
+	 *	If the client network isn't global (not tied to a
+	 *	virtual server), then ensure that this clients server
+	 *	is the same as the enclosing networks virtual server.
+	 */
+	if (master->server &&
+	     (strcmp(master->server, c->server) != 0)) {
+		DEBUG("- Cannot add client %s: Virtual server %s is not the same as the virtual server for the network.",
+		      ip_ntoh(&c->ipaddr,
+			      buffer, sizeof(buffer)),
+		      c->server);
+
+		goto error;
+	}
+
+	if (!client_add(clients, c)) {
+		DEBUG("- Cannot add client %s: Internal error",
+		      ip_ntoh(&c->ipaddr,
+			      buffer, sizeof(buffer)));
+
+		goto error;
+	}
+
+	/*
+	 *	Initialize the remaining fields.
+	 */
+	c->dynamic = TRUE;
+	c->lifetime = master->lifetime;
+	c->created = time(NULL);
+	c->longname = strdup(c->shortname);
+
+	DEBUG("- Added client %s with shared secret %s",
+	      ip_ntoh(&c->ipaddr, buffer, sizeof(buffer)),
+	      c->secret);
+
+	return 1;
+
+ error:
+	client_free(c);
+	return 0;
+}
+
+
+RADCLIENT *client_create(RADCLIENT_LIST *clients, REQUEST *request)
+{
+	int i, *pi;
+	char **p;
+	RADCLIENT *c;
+	char buffer[128];
+
+	if (!clients || !request) return NULL;
+
+	c = rad_malloc(sizeof(*c));
+	memset(c, 0, sizeof(*c));
+	c->cs = request->client->cs;
+	c->ipaddr.af = AF_UNSPEC;
+
+	for (i = 0; dynamic_config[i].name != NULL; i++) {
+		DICT_ATTR *da;
+		VALUE_PAIR *vp;
+
+		da = dict_attrbyname(dynamic_config[i].name);
+		if (!da) {
+			DEBUG("- Cannot add client %s: attribute \"%s\"is not in the dictionary",
+			      ip_ntoh(&request->packet->src_ipaddr,
+				      buffer, sizeof(buffer)),
+			      dynamic_config[i].name);
+		error:
+			client_free(c);
+			return NULL;
+		}
+
+		vp = pairfind(request->config_items, da->attr);
+		if (!vp) {
+			/*
+			 *	Not required.  Skip it.
+			 */
+			if (!dynamic_config[i].dflt) continue;
+			
+			DEBUG("- Cannot add client %s: Required attribute \"%s\" is missing.",	
+			      ip_ntoh(&request->packet->src_ipaddr,
+				      buffer, sizeof(buffer)),
+			      dynamic_config[i].name);
+			goto error;
+		}
+
+		switch (dynamic_config[i].type) {
+		case PW_TYPE_IPADDR:
+			c->ipaddr.af = AF_INET;
+			c->ipaddr.ipaddr.ip4addr.s_addr = vp->vp_ipaddr;
+			c->prefix = 32;
+			break;
+
+		case PW_TYPE_IPV6ADDR:
+			c->ipaddr.af = AF_INET6;
+			c->ipaddr.ipaddr.ip6addr = vp->vp_ipv6addr;
+			c->prefix = 128;
+			break;
+
+		case PW_TYPE_STRING_PTR:
+			p = (char **) ((char *) c + dynamic_config[i].offset);
+			if (*p) free(*p);
+			*p = strdup(vp->vp_strvalue);
+			break;
+
+		case PW_TYPE_BOOLEAN:
+			pi = (int *) ((char *) c + dynamic_config[i].offset);
+			*pi = vp->vp_integer;
+			break;
+
+		default:
+			goto error;
+		}
+	}
+
+	if (c->ipaddr.af == AF_UNSPEC) {
+		DEBUG("- Cannot add client %s: No IP address was specified.",
+		      ip_ntoh(&request->packet->src_ipaddr,
+			      buffer, sizeof(buffer)));
+
+		goto error;
+	}
+
+	if (fr_ipaddr_cmp(&request->packet->src_ipaddr, &c->ipaddr) != 0) {
+		char buf2[128];
+
+		DEBUG("- Cannot add client %s: IP address %s do not match",
+		      ip_ntoh(&request->packet->src_ipaddr,
+			      buffer, sizeof(buffer)),
+		      ip_ntoh(&c->ipaddr,
+			      buf2, sizeof(buf2)));		      
+		goto error;
+	}
+
+	if (!client_validate(clients, request->client, c)) {
+		return NULL;
+	}
+
+	return c;
+}
+
+/*
+ *	Read a client definition from the given filename.
+ */
+RADCLIENT *client_read(const char *filename, int in_server)
+{
+	const char *p;
+	RADCLIENT *c;
+	CONF_SECTION *cs;
+	char buffer[256];
+
+	if (!filename) return NULL;
+
+	cs = cf_file_read(filename);
+	if (!cs) return NULL;
+
+	c = client_parse(cf_section_sub_find(cs, "client"), in_server);
+
+	p = strrchr(filename, FR_DIR_SEP);
+	if (p) {
+		p++;
+	} else {
+		p = filename;
+	}
+
+	/*
+	 *	Additional validations
+	 */
+	ip_ntoh(&c->ipaddr, buffer, sizeof(buffer));
+	if (strcmp(p, buffer) != 0) {
+		DEBUG("Invalid client definition in %s: IP address %s does not match name %s", filename, buffer, p);
+		client_free(c);
+		return NULL;
+	}
+
+
+
+	return c;
+}
+#endif
