@@ -5,7 +5,7 @@
  *		write a decent parser. I know how to do that, really :)
  *		miquels@cistron.nl
  *
- * Version:	$Id: conffile.c,v 1.197 2008/01/09 22:37:58 aland Exp $
+ * Version:	$Id$
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: conffile.c,v 1.197 2008/01/09 22:37:58 aland Exp $")
+RCSID("$Id$")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/rad_assert.h>
@@ -447,6 +447,42 @@ static CONF_SECTION *cf_section_alloc(const char *name1, const char *name2,
 	return cs;
 }
 
+/*
+ *	Replace pair in a given section with a new pair,
+ *	of the given value.
+ */
+int cf_pair_replace(CONF_SECTION *cs, CONF_PAIR *cp, const char *value)
+{
+	CONF_PAIR *newp;
+	CONF_ITEM *ci, *cn, **last;
+
+	newp = cf_pair_alloc(cp->attr, value, cp->operator, cp->value_type,
+			     cs);
+	if (!newp) return -1;
+
+	ci = cf_pairtoitem(cp);
+	cn = cf_pairtoitem(newp);
+
+	/*
+	 *	Find the old one from the linked list, and replace it
+	 *	with the new one.
+	 */
+	for (last = &cs->children; (*last) != NULL; last = &(*last)->next) {
+		if (*last == ci) {
+			cn->next = (*last)->next;
+			*last = cn;
+			ci->next = NULL;
+			break;
+		}
+	}
+
+	rbtree_deletebydata(cs->pair_tree, ci);
+
+	rbtree_insert(cs->pair_tree, cn);
+
+	return 0;
+}
+
 
 /*
  *	Add an item to a configuration section.
@@ -564,6 +600,8 @@ CONF_ITEM *cf_reference_item(const CONF_SECTION *parentcs,
 		 *	"foo.bar.baz" means "from the root"
 		 */
 	} else if (strchr(p, '.') != NULL) {
+		if (!parentcs) goto no_such_item;
+
 		cs = parentcs;
 	}
 
@@ -634,7 +672,7 @@ CONF_ITEM *cf_reference_item(const CONF_SECTION *parentcs,
 	/*
 	 *	"foo" is "in the current section, OR in main".
 	 */
-	if ((p == name) && (cs != parentcs)) {
+	if ((p == name) && (parentcs != NULL) && (cs != parentcs)) {
 		cs = parentcs;
 		goto retry;
 	}
@@ -728,6 +766,11 @@ static const char *cf_expand_variables(const char *cf, int *lineno,
 			 *  Substitute the value of the variable.
 			 */
 			cp = cf_itemtopair(ci);
+			if (!cp->value) {
+				radlog(L_ERR, "%s[%d]: Reference \"%s\" has no value",
+				       cf, *lineno, input);
+				return NULL;
+			}
 			strcpy(p, cp->value);
 			p += strlen(p);
 			ptr = end + 1;
@@ -806,10 +849,10 @@ int cf_item_parse(CONF_SECTION *cs, const char *name,
 	char **q;
 	const char *value;
 	fr_ipaddr_t ipaddr;
-	const CONF_PAIR *cp;
+	const CONF_PAIR *cp = NULL;
 	char ipbuf[128];
 
-	cp = cf_pair_find(cs, name);
+	if (cs) cp = cf_pair_find(cs, name);
 	if (cp) {
 		value = cp->value;
 
@@ -1250,7 +1293,44 @@ static int cf_section_read(const char *filename, int *lineno, FILE *fp,
 		}
 
 		ptr = cbuf = buf;
-		t1 = gettoken(&ptr, buf1, sizeof(buf1));
+
+		/*
+		 *	The parser is getting to be evil.
+		 */
+		while ((*ptr == ' ') || (*ptr == '\t')) ptr++;
+
+		if (((ptr[0] == '%') && (ptr[1] == '{')) ||
+		    (ptr[0] == '`')) {
+			int hack;
+
+			if (ptr[0] == '%') {
+				hack = rad_copy_variable(buf1, ptr);
+			} else {
+				hack = rad_copy_string(buf1, ptr);
+			}
+			if (hack < 0) {
+				radlog(L_ERR, "%s[%d]: Invalid expansion: %s",
+				       filename, *lineno, ptr);
+				return -1;
+			}
+
+			t1 = T_BARE_WORD;
+			ptr += hack;
+
+			t2 = gettoken(&ptr, buf2, sizeof(buf2));
+			switch (t2) {
+			case T_EOL:
+			case T_HASH:
+				goto do_bare_word;
+				
+			default:
+				radlog(L_ERR, "%s[%d]: Invalid expansion: %s",
+				       filename, *lineno, ptr);
+				return -1;
+			}
+		} else {
+			t1 = gettoken(&ptr, buf1, sizeof(buf1));
+		}
 
 		/*
 		 *	The caller eats "name1 name2 {", and calls us
@@ -1415,6 +1495,7 @@ static int cf_section_read(const char *filename, int *lineno, FILE *fp,
 		switch (t2) {
 		case T_EOL:
 		case T_HASH:
+		do_bare_word:
 			t2 = T_OP_EQ;
 			value = NULL;
 			goto do_set;
@@ -1727,7 +1808,7 @@ const char *cf_pair_value(CONF_PAIR *pair)
 /*
  *	Copied here for error reporting.
  */
-extern void librad_log(const char *, ...);
+extern void fr_strerror_printf(const char *, ...);
 
 /*
  * Turn a CONF_PAIR into a VALUE_PAIR
@@ -1735,33 +1816,26 @@ extern void librad_log(const char *, ...);
  */
 VALUE_PAIR *cf_pairtovp(CONF_PAIR *pair)
 {
-	DICT_ATTR *da;
 	VALUE_PAIR *vp;
 
 	if (!pair) {
-		librad_log("Internal error");
-		return NULL;
-	}
-
-	da = dict_attrbyname(pair->attr);
-	if (!da) {
-		librad_log("Unknown attribute %s", pair->attr);
+		fr_strerror_printf("Internal error");
 		return NULL;
 	}
 
 	if (!pair->value) {
-		librad_log("No value given for attribute %s", pair->attr);
+		fr_strerror_printf("No value given for attribute %s", pair->attr);
 		return NULL;
 	}
 
-	vp = pairalloc(da);
+	/*
+	 *	pairmake handles tags.  pairalloc() doesn't.
+	 */
+	vp = pairmake(pair->attr, NULL, pair->operator);
 	if (!vp) {
-		librad_log("Out of memory");
 		return NULL;
 	}
 
-	vp->operator = pair->operator;
-
 	if (pair->value_type == T_BARE_WORD) {
 		if ((vp->type == PW_TYPE_STRING) && 
 		    (pair->value[0] == '0') && (pair->value[1] == 'x')) {
@@ -2360,25 +2434,25 @@ void cf_log_err(CONF_ITEM *ci, const char *fmt, ...)
 }
 
 
-void cf_log_info(UNUSED CONF_SECTION *cs, const char *fmt, ...)
+void cf_log_info(CONF_SECTION *cs, const char *fmt, ...)
 {
 	va_list ap;
 
 	va_start(ap, fmt);
-	if (debug_flag > 1 && cf_log_config) vradlog(L_DBG, fmt, ap);
+	if (debug_flag > 1 && cf_log_config && cs) vradlog(L_DBG, fmt, ap);
 	va_end(ap);
 }
 
 /*
  *	Wrapper to simplify the code.
  */
-void cf_log_module(UNUSED CONF_SECTION *cs, const char *fmt, ...)
+void cf_log_module(CONF_SECTION *cs, const char *fmt, ...)
 {
 	va_list ap;
 	char buffer[256];
 
 	va_start(ap, fmt);
-	if (debug_flag > 1 && cf_log_modules) {
+	if (debug_flag > 1 && cf_log_modules && cs) {
 		vsnprintf(buffer, sizeof(buffer), fmt, ap);
 
 		radlog(L_DBG, " Module: %s", buffer);
@@ -2386,6 +2460,12 @@ void cf_log_module(UNUSED CONF_SECTION *cs, const char *fmt, ...)
 	va_end(ap);
 }
 
+const CONF_PARSER *cf_section_parse_table(CONF_SECTION *cs)
+{
+	if (!cs) return NULL;
+
+	return cs->variables;
+}
 
 #if 0
 /*
